package org.lsh.webservice.ui.config;

import com.google.gson.TypeAdapter;
import com.google.gson.stream.JsonReader;
import com.google.gson.stream.JsonWriter;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.util.Date;
import java.util.GregorianCalendar;

public class XMLGregorianCalendarTypeAdapter extends TypeAdapter<XMLGregorianCalendar> {
    @Override
    public void write(JsonWriter jsonWriter, XMLGregorianCalendar value) throws IOException {
        // 将XMLGregorianCalendar转换为字符串写入JSON
        jsonWriter.value(value.toXMLFormat());
    }

    @Override
    public XMLGregorianCalendar read(JsonReader in) throws IOException {
        // 从JSON读取字符串并转换为XMLGregorianCalendar
        try {
            String dateString = in.nextString();
            GregorianCalendar calendar = new GregorianCalendar();
            Date date = DateFormat.getDateInstance().parse(dateString);
            calendar.setTime(date);
            return DatatypeFactory.newInstance().newXMLGregorianCalendar(calendar);
        } catch (ParseException | DatatypeConfigurationException e) {
            throw new IOException("Failed to parse XMLGregorianCalendar", e);
        }
    }
}
