package org.lsh.webservice.ui.controller;

import cn.hutool.core.collection.CollectionUtil;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.lsh.webservice.ui.common.Result;
import org.lsh.webservice.ui.entity.FormField;
import org.lsh.webservice.ui.entity.FormItemV2;
import org.lsh.webservice.ui.entity.FormV2;
import org.lsh.webservice.ui.entity.dto.ServerInfo;
import org.lsh.webservice.ui.entity.tree.Operation;
import org.lsh.webservice.ui.entity.tree.Project;
import org.lsh.webservice.ui.entity.tree.Webservice;
import org.lsh.webservice.ui.entity.vo.DynamicWebVO;
import org.lsh.webservice.ui.entity.vo.OperationQueryVO;
import org.lsh.webservice.ui.entity.vo.Wsdl;
import org.lsh.webservice.ui.mapper.OperationMapper;
import org.lsh.webservice.ui.mapper.ProjectMapper;
import org.lsh.webservice.ui.mapper.ServerInfoMapper;
import org.lsh.webservice.ui.mapper.WebserviceMapper;
import org.lsh.webservice.ui.utils.ExplainWebservice;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

@Controller("MainController")
public class MainController {

    @Resource
    ProjectMapper projectMapper;
    @Resource
    WebserviceMapper webserviceMapper;
    @Resource
    OperationMapper operationMapper;
    @Resource
    ServerInfoMapper serverInfoMapper;
    @Resource
    ObjectMapper objectMapper;

    @PostMapping("/add-webservice")
    @ResponseBody
    public Result<FormV2> addWsdl(
            @RequestBody Wsdl wsdl
    ){
        Project project = projectMapper.findById(wsdl.getProjectId());
        if(project == null){
            return Result.failed("所选项目ID不存在！");
        }
        FormV2 formV2 = null;
        try {
            formV2 = ExplainWebservice.explainWsdlV2(wsdl.getWsdl());
        }catch (Exception e){
            return Result.failed("解析wsdl失败：" + e.getMessage());
        }

        Webservice webservice = new Webservice();
        webservice.setName(formV2.getServerName());
        webservice.setWsdl(wsdl.getWsdl());
        webservice.setProjectId(project.getId());
        webserviceMapper.insert(webservice);

        List<FormItemV2> formItemV2List = formV2.getFormItemV2List();
        if(CollectionUtil.isNotEmpty(formItemV2List)){
            for (FormItemV2 formItemV2 : formItemV2List) {
                Operation operation = new Operation();
                operation.setWebserviceId(webservice.getId());
                operation.setName(formItemV2.getOperation());
                try {
                    operation.setInput(objectMapper.writeValueAsString(formItemV2.getInput()));
                    operation.setOutput(objectMapper.writeValueAsString(formItemV2.getOutput()));
                } catch (JsonProcessingException e) {
                    return Result.failed(e.getMessage());
                }
                operationMapper.insert(operation);
            }
        }

        return Result.ok();
    }

    @PostMapping("/explain-wsdl")
    @ResponseBody
    public Result<FormV2> receiveCheckPlan(
            @RequestBody Webservice webservice
    ){
        System.out.println("wsdl:" + webservice.getWsdl());
        ServerInfo serverInfo = serverInfoMapper.findByWebserviceId(webservice.getId());
        if(serverInfo != null){
            String formJson = serverInfo.getFormJson();
            FormV2 formV2 = null;
            try {
                formV2 = objectMapper.readValue(formJson, FormV2.class);
            } catch (JsonProcessingException e) {
                return Result.failed(e.getMessage());
            }
            return Result.ok(formV2);
        }
        return Result.ok();
    }

    @PostMapping("/query-projects")
    @ResponseBody
    public Result<List<Project>> queryProjects(
            @RequestParam(value = "loadWebservice",defaultValue = "true")String loadWebservice,
            @RequestParam(value = "loadOperation",defaultValue = "true")String loadOperation
            ){
        //
        List<Project> projects = projectMapper.findAll();
        if(loadWebservice.equals("true")) {
            for (Project project : projects) {
                List<Webservice> webserviceList = webserviceMapper.findByProjectId(project.getId());
                project.setChildren(webserviceList);
                if (CollectionUtil.isNotEmpty(webserviceList) && loadOperation.equals("true")) {
                    for (Webservice webservice : webserviceList) {
                        List<Operation> operationList = operationMapper.findByWebserviceId(webservice.getId());
                        webservice.setChildren(operationList);
                    }
                }
            }
        }
        return Result.ok(projects);
    }

    @GetMapping("/query-operation")
    @ResponseBody
    public Result<OperationQueryVO> queryOperation(
            @RequestParam(value = "webserviceId",required = false)Integer webserviceId,
            @RequestParam(value = "operationName",required = false)String operationName,
            @RequestParam("operationId")Integer operationId
    ){
        OperationQueryVO operationQueryVO = new OperationQueryVO();
        Operation operation = operationMapper.findById(operationId);
        if(operation != null) {
            Webservice webservice = webserviceMapper.findById(operation.getWebserviceId());
            operationQueryVO.setOperation(operation.getName());
            operationQueryVO.setId(operation.getId());
            try {
                operationQueryVO.setInput(objectMapper.readValue(operation.getInput(), FormField.class));
                operationQueryVO.setOutput(objectMapper.readValue(operation.getOutput(), FormField.class));
            } catch (JsonProcessingException e) {
                return Result.failed(e.getMessage());
            }
            if(webservice != null){
                operationQueryVO.setWsdl(webservice.getWsdl());
                operationQueryVO.setWebserviceName(webservice.getName());
            }
        }
        return Result.ok(operationQueryVO);
    }

    @PostMapping("/invoke")
    @ResponseBody
    public Result<FormField> invoke(
            @RequestBody DynamicWebVO webservice
    ){
        try {
            FormField formField = ExplainWebservice.dynamicInvoke(webservice.getWsdl()
                    , webservice.getMethod()
                    , webservice.getInput()
                    , webservice.getOutput()
                    , webservice.getJson());
            return Result.ok(formField);
        } catch (Exception e) {
            e.printStackTrace();
            return Result.failed("调用发生了异常~"+e.getMessage());
        }
    }

    @RequestMapping("/index")
    public String hello(){
        return "index";  // index.html页面
    }
}
