package org.lsh.webservice.ui.utils;

import cn.hutool.core.collection.CollectionUtil;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.JavaType;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.PropertyNamingStrategy;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import org.apache.cxf.endpoint.Client;
import org.apache.cxf.endpoint.Endpoint;
import org.apache.cxf.jaxws.endpoint.dynamic.JaxWsDynamicClientFactory;
import org.apache.cxf.service.model.*;
import org.lsh.webservice.ui.config.XMLGregorianCalendarTypeAdapter;
import org.lsh.webservice.ui.entity.FormItemV2;
import org.lsh.webservice.ui.entity.FormV2;
import org.lsh.webservice.ui.entity.FormField;
import org.lsh.webservice.ui.entity.MyParameterizedType;

import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;
import java.lang.reflect.Field;
import java.lang.reflect.Type;
import java.util.*;

public class ExplainWebservice {

    public static FormV2 explainWsdlV2(String wsdlUrl){
        FormV2 formV2 = new FormV2();
        List<FormItemV2> itemV2s = new ArrayList<>();
        formV2.setFormItemV2List(itemV2s);
        Map<String,FormField> messageMap = new HashMap<>();
        // 创建动态客户端
        JaxWsDynamicClientFactory factory = JaxWsDynamicClientFactory.newInstance();
        Client client = factory.createClient(wsdlUrl);
        /**endpoint据说为http://172.20.29.51:8180/uniplatform/service/UniNewOrderDataService
         * 不过toString方法打印的为{},有点奇怪,不过getEndpointInfo打印的为BindingQName，ServiceQName，QName*/
        Endpoint endpoint = client.getEndpoint();
        /**获取Service*/
        ServiceInfo serviceInfo = endpoint.getService().getServiceInfos().get(0);
        formV2.setServerName(serviceInfo.getName().getLocalPart());
        /**创建Service*/
        Collection<BindingInfo> bindings = serviceInfo.getBindings();
//       portType
//       对于某个访问入口点类型所支持的操作的抽象集合； ----》服务端SEI
//　　    指出了这个WebService所有支持的操作，就是说有哪些方法可供调用。

        Map<QName, MessageInfo> messages = serviceInfo.getMessages();
//        message 指明一个操作所用到的数据类型，通信消息的数据结构的抽象类型化定义。
//        HelloWorldSoapIn是指HelloWorld的输入操作用到的数据类型，
//        HelloWorldSoapOut是指HelloWorld的输出操作用到的数据类型。二者的element元素指出了与types中对应到的具体类型。
        messages.forEach((key,value)->{
            List<MessagePartInfo> messageParts = value.getMessageParts();
            for (MessagePartInfo messagePart : messageParts) {
                Class<?> typeClass = messagePart.getTypeClass();
                String field = "";
                if(messagePart.getTypeQName()!=null){
                    field = messagePart.getTypeQName().getLocalPart();
                }else if(messagePart.getElementQName()!=null){
                    field = messagePart.getElementQName().getLocalPart();
                }else if(messagePart.getConcreteName()!=null){
                    field = messagePart.getConcreteName().getLocalPart();
                }

                try {
                    Class<?> aClass = Thread.currentThread().getContextClassLoader().loadClass(typeClass.getName());
                    FormField complex = new FormField();
                    complex.setFieldName(field);
                    List<FormField> formFields = new ArrayList<>();
                    complex.setFields(formFields);
                    complex.setTypeName(aClass.getTypeName());
                    Field[] declaredFields = aClass.getDeclaredFields();
                    System.out.println("====>"+aClass);
                    for (Field declaredField : declaredFields) {
                        System.out.println("declaredField:" + declaredField);
                        org.lsh.webservice.ui.utils.ExplainField.explain(null,declaredField, formFields);
                    }
                    //System.out.println("解析表单结果:" + JSON.toJSONString(complex));
                    messageMap.put(aClass.getSimpleName(),complex);
                } catch (ClassNotFoundException | IllegalAccessException e) {
                    e.printStackTrace();
                }
            }
        });

//       Binding-特定端口类型的具体协议和数据格式规范的绑定。----》对应SEI的实现
//　　    soap:binding元素的transport指明传输协议，这里是http协议。
//　　    operation 指明要暴露给外界调用的操作。
//　　    use属性指定输入输出的编码方式。
        if(CollectionUtil.isNotEmpty(bindings)){
            for (BindingInfo binding : bindings) {
                for (BindingOperationInfo operation : binding.getOperations()) {
                    String function = operation.getName().getLocalPart();
                    String inputTypeName = "";
                    String outputTypeName = "";

                    FormItemV2 itemV2 = new FormItemV2();
                    itemV2.setOperation(function);

                    OperationInfo operationInfo = operation.getOperationInfo();
                    if(operation.isUnwrapped()){
                        operationInfo = operation.getUnwrappedOperation().getOperationInfo();
                    }
                    BindingMessageInfo messageInfo = null;
                    if(operation.isUnwrapped()){
                        messageInfo = operation.getUnwrappedOperation().getInput();
                    }else{
                        messageInfo = operation.getWrappedOperation().getInput();
                    }

                    List<MessagePartInfo> messageParts = messageInfo.getMessageInfo().getMessageParts();
                    StringBuilder params = new StringBuilder();
                    for (MessagePartInfo messagePart : messageParts) {
                        String typeName = messagePart.getTypeClass().getTypeName();
                        String param = messagePart.getName().getLocalPart();
                        if(params.length()>0){
                            params.append(",");
                        }
                        params.append(typeName+" "+param);
                    }
                    System.out.println("function==================>:" + function + "(" + params.toString()+")");

                    List<MessagePartInfo> inputMessageParts = messageInfo.getMessageParts();
                    for (MessagePartInfo inputMessagePart : inputMessageParts) {
                        inputTypeName = inputMessagePart.getTypeClass().getSimpleName();
                        String name = inputMessagePart.getName().getLocalPart();
                        // 如果是参数命名，那么再往下，获取实际传参
                        FormField input = messageMap.get(inputTypeName);
//                        if("parameters".equals(name) && input.getFields().size()>0){
//                            input = input.getFields().get(0);
//                        }
                        itemV2.setInput(input);
                    }
                    List<MessagePartInfo> outputMessageParts = operationInfo.getOutput().getMessageParts();
                    for (MessagePartInfo outputMessagePart : outputMessageParts) {
                        outputTypeName = outputMessagePart.getTypeClass().getSimpleName();
                        String name = outputMessagePart.getName().getLocalPart();
                        // 如果是参数命名，那么再往下，获取实际传参
                        FormField output = messageMap.get(outputTypeName);
//                        if("parameters".equals(name) && output.getFields().size()>0){
//                            output = output.getFields().get(0);
//                        }
                        itemV2.setOutput(output);
                    }
                    itemV2s.add(itemV2);
                }
            }
        }
        return formV2;
    }

    public static FormField dynamicInvoke(String wsdl
            ,String method
            ,FormField input
            ,FormField output
            ,String json) throws Exception {
        ObjectMapper objectMapper = new ObjectMapper();
        // 创建动态客户端
        JaxWsDynamicClientFactory factory = JaxWsDynamicClientFactory.newInstance();
        Client client = factory.createClient(wsdl);
        String inputClassName = ExplainField.switchToObject(input.getTypeName(),input.getGenericType());
        Class<?> inputClass = Thread.currentThread().getContextClassLoader().loadClass(inputClassName);
        // 如果输入值是集合，那么需要将集合转化为对应类型
        JavaType inputJavaType = objectMapper.getTypeFactory().constructCollectionType(List.class, inputClass);
        MyParameterizedType type = new MyParameterizedType(List.class, new Type[]{inputClass});

        String outputClassName = ExplainField.switchToObject(output.getTypeName(),output.getGenericType());
        Class<?> outputClass = Thread.currentThread().getContextClassLoader().loadClass(outputClassName);
        JavaType outputJavaType = objectMapper.getTypeFactory().constructCollectionType(List.class, outputClass);


        Object request = null;
        if(ExplainField.isList(input.getTypeName())){
            objectMapper.disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES);
            //objectMapper.setPropertyNamingStrategy(PropertyNamingStrategy.SNAKE_CASE);
            objectMapper.configure(DeserializationFeature.FAIL_ON_IGNORED_PROPERTIES,false);
//            request =  objectMapper.readValue(json, inputJavaType);
            Gson gson = new GsonBuilder()
                    .registerTypeAdapter(XMLGregorianCalendar.class, new XMLGregorianCalendarTypeAdapter())
                    .create();
            request =  gson.fromJson(json, type);
        }else{
            request =  objectMapper.readValue(json, inputClass);
        }
        Object[] invoke = client.invoke(method,request);
        FormField complex = new FormField();

        if(invoke != null && invoke.length>0){
            String responseJson = objectMapper.writeValueAsString(invoke[0]);
            System.out.println("=====response======>"+responseJson);
            Object response = null;
            if(ExplainField.isList(output.getTypeName())){
                response =  objectMapper.readValue(responseJson, outputJavaType);
                System.out.println("=====response======>"+response);
                List<FormField> formFields = new ArrayList<>();
                complex.setFieldName(output.getFieldName());
                complex.setFields(formFields);
                complex.setTypeName(output.getTypeName());
                complex.setGenericType(output.getGenericType());

                // 如果本身就是集合，那么字段名称就是下标
                List list = (List) response;
                for (int i = 0; i < list.size(); i++) {
                    Object o1 = list.get(i);
                    List<FormField> indexFormFields = new ArrayList<>();
                    FormField indexForm = new FormField();
                    indexForm.setTypeName(o1.getClass().getTypeName());
                    indexForm.setFields(indexFormFields);
                    indexForm.setFieldName(i+"");
                    Field[] declaredFields = o1.getClass().getDeclaredFields();
                    for (Field declaredField : declaredFields) {
                        org.lsh.webservice.ui.utils.ExplainField.explain(o1,declaredField, indexFormFields);
                    }
                    formFields.add(indexForm);
                }
            }else{
                response = objectMapper.readValue(responseJson, outputClass);
                System.out.println("=====response======>"+response);
                List<FormField> formFields = new ArrayList<>();
                complex.setFields(formFields);
                complex.setTypeName(outputClassName);
                complex.setFieldName(output.getFieldName());
                Field[] declaredFields = outputClass.getDeclaredFields();
                for (Field declaredField : declaredFields) {
                    org.lsh.webservice.ui.utils.ExplainField.explain(response,declaredField, formFields);
                }
            }
        }
        return complex;
    }


}
